/*******************************************************************************

   File:          searchbarext.c

   Description:   A searchbar plugin for K-Meleon browser.

   Copyright:     (C) 2008 TEK

*******************************************************************************/


/*******************************************************************************
   Includes
*******************************************************************************/

#ifdef __MINGW32__
#	define _WIN32_IE 0x0500
#	include <windows.h>
#include <Windowsx.h>
#	include <commctrl.h>
#endif

#define KMELEON_PLUGIN_EXPORTS
#include "kmeleon_plugin.h"
#include "utf.h"              //utf8_from_utf16, utf8_from_ansi

#include <stdio.h>
#include <tchar.h>
//#include "afxwin.h"
#include "searchbarext.h"
#include "droptarget.h"

/*******************************************************************************
   Macros and constants
*******************************************************************************/

#define PLUGIN_NAME                          "SearchBarExt"
#define PLUGIN_VERSION                       "0.4.0"

#define TOOLBAND_NAME                        "SearchBar Ext"

#define OPEN_IN_CUR_TAB                      0x01
#define OPEN_IN_NEW_TAB                      0x02
#define OPEN_IN_NEW_TAB_BG                   0x03
#define OPEN_IN_NEW_WIN                      0x04
#define OPEN_IN_NEW_WIN_BG                   0x05



/*******************************************************************************
   Function prototypes
*******************************************************************************/

// Plugin entry
BOOL APIENTRY    DllMain ( HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved) { return TRUE; }

// Controls callback functions
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK EditProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK ToolbarProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL APIENTRY    DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

// API messages
long DoMessage(const char *to, const char *from, const char *subject, long data1, long data2);

//virtual bool PreTranslateMessage(MSG * pMsg);

// plugin functions
int  LoadPlugin();
int  InitPlugin();
void QuitPlugin();

// windows functions
void CreateWnd(HWND hWnd);
void DestroyWnd(HWND hWnd);
void DoRebar(HWND rebarWnd);

// unused functions
void Config(HWND parent){;}
void DoMenu(HMENU menu, char *param){;}
int  DoAccel(char *param){return true;}
int  GetConfigFiles(configFileType **configFiles){return true;}

//
void Search(const char * sTerm);
void OpenURL(char *url, int open);

//
void RefreshLayersEdit(HWND hWnd, CHAR *sText);
void RebuildBands(UINT uSize);

HFONT hFont = NULL;

/*******************************************************************************
   Global variables and defines
*******************************************************************************/

// controls & associates
typedef struct WNDCXT {
	HWND hwndToolbar;
	HWND hwndEdit;
	BOOL bText;
	IDropTarget *pDropTarget;
} WNDCXT, *PTRWNDCXT;

// Force unicode at compilation
#ifdef _UNICODE
BOOL g_bUnicode = TRUE;
#else
BOOL g_bUnicode = FALSE;
#endif

HMODULE hmodUnicode = NULL;

//
WNDPROC	g_wpEditProc;     // Edit callback proc
WNDPROC	g_wpToolbarProc;  // Toolbar callback proc
void * KMeleonWndProc;    // KM main window callback proc

// Command IDs
UINT idCmdEdit;
UINT idCmdSearch;
UINT idCmdEngines;
UINT idCmdSettings;
UINT idCmdClear;

char CurrentPath[MAX_PATH];
bool focus = false;

//
kmeleonPlugin kPlugin =
{
   KMEL_PLUGIN_VER,
   PLUGIN_NAME,
   DoMessage
};


/******************************************************************************
   Win32 API UTF8 mapping functions
 ******************************************************************************/

// change some features from original functions
// -return treu/false instead of string length
// -no nMaxCount as an entry
// -lpString is allocated in the function--> need to be free after used
char * GetWindowTextUTF8(HWND hWnd)
{
	if (g_bUnicode)
	{
		WCHAR sTextW[1024];
		GetWindowTextW(hWnd, sTextW, sizeof(sTextW)/sizeof(WCHAR));
		return utf8_from_utf16(sTextW);
	}
	else
	{
		char sTextA[1024];
		GetWindowTextA(hWnd, sTextA, sizeof(sTextA));
		return utf8_from_ansi(sTextA);
	}
}

LONG_PTR SetWindowLongPtrUTF8(HWND hWnd, int nIndex, LONG_PTR dwNewLong)
{
	return (g_bUnicode)
		? SetWindowLongPtrW(hWnd, GWLP_WNDPROC, dwNewLong)
		: SetWindowLongPtrA(hWnd, GWLP_WNDPROC, dwNewLong);
}

LRESULT CallWindowProcUTF8(WNDPROC lpPrevWndFunc, HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	return (g_bUnicode)
		? CallWindowProcW(lpPrevWndFunc, hWnd, Msg, wParam, lParam)
		: CallWindowProcW(lpPrevWndFunc, hWnd, Msg, wParam, lParam);
}


/******************************************************************************
   DoMessage

   note: _Close_ is replaced by _Destroy_ because when the user exit using
		 "File -> Exit K-Meleon" or with accelerate key Ctrl+Alt+F4, the _Close_
		 message is not send to the plugin (?!)
 ******************************************************************************/
long DoMessage(const char *to, const char *from, const char *subject, long data1, long data2)
{
	if (to[0] == '*' || stricmp(to, kPlugin.dllname) == 0)
	{
		if (stricmp(subject, "Load") == 0) {
			LoadPlugin();
		}
		else if (stricmp(subject, "Init") == 0) {
			InitPlugin();
		}
		else if (stricmp(subject, "Quit") == 0) {
			QuitPlugin();
		}
		else if (stricmp(subject, "Create") == 0) {
			CreateWnd((HWND)data1);
		}
		else if (stricmp(subject, "Destroy") == 0) {
			DestroyWnd((HWND)data1);
		}
		else if (stricmp(subject, "DoRebar") == 0) {
			DoRebar((HWND)data1);
		}
		else return 0;

		return 1;
	}

   return 0;
}


/******************************************************************************
   LoadPlugin
   - called 1 time
   - before inititialisation
 ******************************************************************************/
int LoadPlugin()
{
	// Reserve command IDs
	idCmdEdit = kPlugin.kFuncs->GetCommandIDs(1);
	idCmdSearch = kPlugin.kFuncs->GetCommandIDs(1);
	idCmdSettings = kPlugin.kFuncs->GetCommandIDs(1);
	idCmdClear = kPlugin.kFuncs->GetCommandIDs(1);
	idCmdEngines = kPlugin.kFuncs->GetCommandIDs(1);

	// This program requires COM
	OleInitialize(0);

	// Check unicode system (we assume that all NT are unicode, 95/98 and below are not)
	// when not forced at compilation
#ifndef _UNICODE
	OSVERSIONINFO osinfo;
	osinfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&osinfo);
	//g_bUnicode = (osinfo.dwPlatformId == VER_PLATFORM_WIN32_NT);
	if(osinfo.dwPlatformId==VER_PLATFORM_WIN32_NT)
	{
		g_bUnicode = TRUE;
	}
	// Check if Windows 95/98 has MSLU installed
	else if(osinfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
	{
		// We assume that DLL is in system directory
		if(GetModuleHandle("unicows.dll"))
		{
			g_bUnicode = TRUE;
		}
		// Install the MSLU
		else if(hmodUnicode = LoadLibraryA("unicows.dll"))
		{
			g_bUnicode = TRUE;
		}
	}
#endif

	return true;
}

/******************************************************************************
   InitPlugin
   - called 1 time
 ******************************************************************************/
int InitPlugin()
{
	return true;
}

/******************************************************************************
   QuitPlugin
   - called 1 time
   - when closing KM
 ******************************************************************************/
void QuitPlugin()
{
	// Shutdown COM
	OleUninitialize();

	// Free loaded DLL
	if(hmodUnicode)
		FreeLibrary(hmodUnicode);
}

/******************************************************************************
   CreateWnd
   - called for each new window
 ******************************************************************************/
void CreateWnd(HWND hWnd)
{
	// Save the window callback function
	KMeleonWndProc = (void *) GetWindowLong(hWnd, GWL_WNDPROC);
	// Set ...
	SetWindowLong(hWnd, GWL_WNDPROC, (LONG)WndProc);

	// Create a new context
   WNDCXT * pcxt = (WNDCXT *)malloc(sizeof(WNDCXT)); //allocate memory
	if(!pcxt) return;

	// Initialise the new context (text management)
	pcxt->bText = FALSE;
	pcxt->hwndToolbar = NULL;
	pcxt->hwndEdit = NULL;

	// Associate the new context to the window
	SetProp(hWnd, _T("WNDCXT"), (HANDLE)(pcxt));
}

/******************************************************************************
   DestroyWnd
   - called for each new window
 ******************************************************************************/
void DestroyWnd(HWND hWnd)
{
	// get context
	PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWnd, _T("WNDCXT"));
	if(pcxt)
	{
		if(pcxt->hwndEdit)
			UnregisterDropWindow(pcxt->hwndEdit, pcxt->pDropTarget);

		RemoveProp(hWnd, _T("WNDCXT"));

		DestroyWindow(pcxt->hwndEdit);
		DestroyWindow(pcxt->hwndToolbar);

		free(pcxt);
	}
}

/******************************************************************************
   DoRebar
   - called for each new window
 ******************************************************************************/
void DoRebar(HWND rebarWnd)
{
	// get rebar parent (should be a window)
	HWND hWnd = GetParent(rebarWnd);

	// get context
	PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWnd, _T("WNDCXT"));
	if(!pcxt) return;

	/********** TOOLBAR **********/

	pcxt->hwndToolbar = CreateWindowEx(0, TOOLBARCLASSNAME, NULL,
							WS_CHILD|WS_VISIBLE|CCS_NODIVIDER | CCS_NOPARENTALIGN | CCS_NORESIZE |TBSTYLE_FLAT | TBSTYLE_TRANSPARENT,
							0, 0, 0, 0, //pos, size
							rebarWnd, 0, kPlugin.hDllInstance, NULL);

	if(!pcxt->hwndToolbar) return;

	// Register the band name and child hwnd
	// --> keep a trace of the BAND positions for the next kmeleon launch
	kPlugin.kFuncs->RegisterBand(pcxt->hwndToolbar, TOOLBAND_NAME, TRUE);

	// Set buttons
	SendMessage(pcxt->hwndToolbar, TB_BUTTONSTRUCTSIZE, (WPARAM)sizeof(TBBUTTON), 0);

	//
	// iBitmap, idCommand, fsState, fsStyle, bReserved[], dwData, iString
	TBBUTTON tbButtons[2];
	ZeroMemory(tbButtons, sizeof(tbButtons));

	tbButtons[0].iBitmap   = 0; //0 = 1st image in the list
	tbButtons[0].idCommand = idCmdSearch;
	tbButtons[0].fsState   = TBSTATE_ENABLED;
	tbButtons[0].fsStyle   =  TBSTYLE_DROPDOWN; //TBSTYLE_BUTTON

	tbButtons[1].iBitmap   = -1;
	tbButtons[1].idCommand = idCmdEdit;
	tbButtons[1].fsState   = 0;
	tbButtons[1].fsStyle   = TBSTYLE_SEP;

	// Insert buttons
	SendMessage(pcxt->hwndToolbar, TB_ADDBUTTONS, sizeof(tbButtons)/sizeof(TBBUTTON), (LPARAM)&tbButtons); //2 <=>

	// Set images list (one image of 16x16 pixels)
	HIMAGELIST hImgList = ImageList_Create( 16, 16, ILC_COLOR32|ILC_MASK, 0, 1 );
	ImageList_AddMasked(hImgList, LoadBitmap(kPlugin.hDllInstance,MAKEINTRESOURCE(IDB_SEARCHBAR)), RGB(255,0,255));

	// Insert images
	SendMessage(pcxt->hwndToolbar, TB_SETIMAGELIST , 0, (LPARAM)hImgList);

	// Button style (with an arrow)
	SendMessage(pcxt->hwndToolbar, TB_SETEXTENDEDSTYLE, 0, TBSTYLE_EX_DRAWDDARROWS);

	// Change the Toolbar window callback procedure
	// --> necessary to change editbox (child) font color
	g_wpToolbarProc = (WNDPROC) SetWindowLongPtr(pcxt->hwndToolbar, GWLP_WNDPROC, (LONG_PTR) ToolbarProc);

	/********** BOUTON **********/

	//
	int iSize = 200;
	kPlugin.kFuncs->GetPreference(PREF_INT, "kmeleon.plugins.searchbarext.size", &iSize, &iSize);

	// Change the SEP width
	TBBUTTONINFO tbButtoninfo;
	ZeroMemory(&tbButtoninfo, sizeof(TBBUTTONINFO));

	tbButtoninfo.cbSize = sizeof(TBBUTTONINFO);
	tbButtoninfo.dwMask = TBIF_SIZE;
	tbButtoninfo.cx     = iSize;

	SendMessage(pcxt->hwndToolbar, TB_SETBUTTONINFO, idCmdEdit, (LPARAM)&tbButtoninfo);

	// Get ID of button which will be modified
	int id = SendMessage(pcxt->hwndToolbar, TB_COMMANDTOINDEX, idCmdEdit, 0);

	// Get coords. of button which will be modified
	RECT rc;
	SendMessage(pcxt->hwndToolbar, TB_GETITEMRECT, id, (LPARAM)&rc);

	/********** EDITBOX **********/
	/*if(g_bUnicode)
	{
		pcxt->hwndEdit = CreateWindowExW( WS_EX_CLIENTEDGE, L"EDIT", L"",
							WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_AUTOHSCROLL |ES_LEFT,
							rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top - 1,
							pcxt->hwndToolbar, (HMENU)idCmdEdit, kPlugin.hDllInstance, NULL );
	}
	else
	{
		pcxt->hwndEdit = CreateWindowExA( WS_EX_CLIENTEDGE, "EDIT", NULL,
							WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_AUTOHSCROLL | ES_LEFT,
							rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top - 1,
							pcxt->hwndToolbar, (HMENU)idCmdEdit, kPlugin.hDllInstance, NULL );
	}*/

	if(g_bUnicode)
	{
		   pcxt->hwndEdit = CreateWindowExW( WS_EX_CLIENTEDGE , L"COMBOBOX" , L""/*(LPCWSTR)NULL*/,
			     			WS_CHILD | WS_VISIBLE | CBS_AUTOHSCROLL | CBS_DROPDOWN | WS_VSCROLL | WS_TABSTOP ,
							rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top + 200,
							pcxt->hwndToolbar, (HMENU)idCmdEdit, kPlugin.hDllInstance, NULL );
	}
	else
	{
		   pcxt->hwndEdit = CreateWindowExA( WS_EX_CLIENTEDGE, "COMBOBOX" , NULL,
			        		WS_CHILD | WS_VISIBLE | CBS_AUTOHSCROLL | CBS_DROPDOWN | WS_VSCROLL | WS_TABSTOP ,
							rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top + 200,
							pcxt->hwndToolbar, (HMENU)idCmdEdit, kPlugin.hDllInstance, NULL );
	}

	if(!pcxt->hwndEdit) return;

	// Set the font
	//HFONT hFont = (HFONT)SendMessage(rebarWnd, WM_GETFONT, 0, 0);

	hFont = (HFONT)SendMessage(pcxt->hwndToolbar, WM_GETFONT, 0, 0);
	/*HFONT hFont , hFontOld;
    hFont = CreateFont(120,
                       0,
                       0,
                       0,
                       FW_NORMAL,
                       FALSE,
                       FALSE,
                       FALSE,
                       ANSI_CHARSET,
                       OUT_DEFAULT_PRECIS,
                       CLIP_DEFAULT_PRECIS,
                       DEFAULT_QUALITY,
                       DEFAULT_PITCH | FF_DONTCARE,
                       "");
   hFontOld = (HFONT)SelectObject(GetDC(pcxt->hwndEdit) , hFont);*/


	if(hFont)
	    SendMessage(pcxt->hwndEdit, WM_SETFONT, (WPARAM)hFont, TRUE);

    /*SelectObject(GetDC(pcxt->hwndEdit) , hFontOld);
    DeleteObject(hFont);
    DeleteObject(hFontOld);*/

	// Initialise content
	char sValue[80];
	kPlugin.kFuncs->GetPreference(PREF_STRING, "kmeleon.general.searchEngineName", sValue, (char*)"");

	SetWindowText(pcxt->hwndEdit, sValue);

	// make the Edit control into a DropTarget
	RegisterDropWindow(pcxt->hwndEdit, &pcxt->pDropTarget);

	// Change the Editbox window callback procedure
	COMBOBOXINFO info = { sizeof(COMBOBOXINFO) };
    GetComboBoxInfo(pcxt->hwndEdit, &info);
    HWND hWnd_cbx = info.hwndCombo;
    HWND hWnd_edit = info.hwndItem;
	g_wpEditProc = (WNDPROC) SetWindowLongPtrUTF8(hWnd_edit , GWLP_WNDPROC, (LONG_PTR) EditProc);

	//Read ini file
	 GetCurrentDirectory(MAX_PATH, CurrentPath);
	 char file[8] = "\\sh.ini";
	 strcat(CurrentPath , file);
	 //MessageBox(hWnd_cbx , CurrentPath , file , 32);  //Test
	int total = GetPrivateProfileInt("Total" , "total" , 0 , (LPCTSTR)CurrentPath);
	if(total > 0)
		{
			 for(int i = 0; i < total; i ++)
			   {
			       //int to LPCTSTR
			       char szBuf[1024];
                   LPCTSTR total_t;
                   sprintf(szBuf , "%d" , i + 1);
                   total_t = LPCTSTR(szBuf);

                   char sItem[1024];
				   GetPrivateProfileString("Item" , total_t , "" , sItem , 1024 , (LPCTSTR)CurrentPath);
                   //MessageBox(hWnd_cbx , total_t , sItem , 32);//Test
			       SendMessage(hWnd_cbx , CB_INSERTSTRING , 0 , (LPARAM)sItem);

			   }
		}


	/********** REBAR BAND **********/

	REBARBANDINFO rbBandinfo = { sizeof(REBARBANDINFO) };

	rbBandinfo.cbSize     = sizeof(REBARBANDINFO);  // Required
	rbBandinfo.fMask      = RBBIM_STYLE | RBBIM_CHILD  | RBBIM_CHILDSIZE | RBBIM_SIZE | RBBIM_IDEALSIZE;
	rbBandinfo.fStyle     = RBBS_CHILDEDGE | RBBS_FIXEDBMP | RBBS_VARIABLEHEIGHT ;
	rbBandinfo.lpText     = NULL;
	rbBandinfo.hwndChild  = pcxt->hwndToolbar;
	rbBandinfo.cxMinChild = rc.right-rc.left +40;
	rbBandinfo.cyMinChild = rc.bottom-rc.top;
	rbBandinfo.cyIntegral = 1;
	rbBandinfo.cyMaxChild = rbBandinfo.cyMinChild;
	rbBandinfo.cxIdeal    = rbBandinfo.cxMinChild;
	rbBandinfo.cx         = rbBandinfo.cxIdeal;

	// Add the band that has the toolbar
	SendMessage(rebarWnd, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbBandinfo);
}


/******************************************************************************
   WndProc
 ******************************************************************************/
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static int nMenuPos;

	// initialize a menu item position before it is displayed
	if (uMsg == WM_INITMENUPOPUP)
	{
		nMenuPos = -1;
	}


	// save the selected menu item
	else if (uMsg == WM_MENUSELECT)
	{
		//LOWORD(wParam)  ==  CMD ID
		//HIWORD(wParam)  ==  ???
		//lParam          ==  MENU HANDLE
		//HIWORD(wParam)=ffff and lParam=0 result menu closed

		if (lParam) //there is a menu handle
		{
			POINT pt;
			if (GetCursorPos(&pt)) // get selected item position
				nMenuPos = MenuItemFromPoint(NULL, (HMENU)lParam, pt);
		}
	}

	else if (uMsg == WM_COMMAND)
	{
		WORD command = LOWORD(wParam);

		// Clic on the search button
		if(LOWORD(wParam) == idCmdSearch)
		{
			focus = true;
		    // get context variable
			PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWnd, _T("WNDCXT"));
			if(pcxt)
			{
				if(pcxt->bText)
				{
					// Get search text (Old)
				    /*char * sText = GetWindowTextUTF8(pcxt->hwndEdit);
					// Perform search
					Search(sText);
					// Free memory
					free(sText);*/

				    COMBOBOXINFO info = { sizeof(COMBOBOXINFO) };
                    GetComboBoxInfo(pcxt->hwndEdit , &info);
				    HWND hWnd_edit = info.hwndItem;
					SendMessage(hWnd_edit , WM_CHAR, VK_RETURN, 0);

				}
				else
				{
					Search("");
				}
			}
		}

		// Check "settings" item command ID
		else if(LOWORD(wParam) == idCmdSettings)
		{
			DialogBox(kPlugin.hDllInstance, MAKEINTRESOURCE(IDD_DLG), hWnd, (DLGPROC)DialogProc);
		}

		// Check "engines" items command ID
		else if(LOWORD(wParam) == idCmdEngines)
		{
			// get context variable
			PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWnd, _T("WNDCXT"));
			if(pcxt)
			{
				char sPref[1024];
				int i = nMenuPos;

				// Get search engine name from selected item
				sprintf(sPref, "kmeleon.plugins.macros.search.engine%d.name", i);
				char sName[1024];
				kPlugin.kFuncs->GetPreference(PREF_STRING, sPref, sName, (char*)"");

				// Set current search engine name
				kPlugin.kFuncs->SetPreference(PREF_STRING, "kmeleon.general.searchEngineName", sName, FALSE);

				// Get search engine from selected item
				sprintf(sPref, "kmeleon.plugins.macros.search.engine%d.url", i);
				char sURL[1024];
				kPlugin.kFuncs->GetPreference(PREF_STRING, sPref, sURL, (char*)"");

				// Set current search engine
				kPlugin.kFuncs->SetPreference(PREF_STRING, "kmeleon.general.searchEngine", sURL, FALSE);

				// Get search term
				CHAR sTerm[1024];
				GetWindowText(pcxt->hwndEdit, sTerm, sizeof(sTerm));

				// Search term is empty and (...)
				if( *sTerm && !pcxt->bText )
				    // Set editbox text to search engine name
					SetWindowText(pcxt->hwndEdit , sName);



				//
				RefreshLayersEdit(hWnd, sName);
			}
		}
	}

	else if (uMsg == WM_NOTIFY)
	{
		LPNMHDR lpnm = (LPNMHDR)lParam;
		LPNMTOOLBAR lpnmTB = (LPNMTOOLBAR)lParam;

		switch(lpnm->code)
		{
			// Clic on the button arrow
			case TBN_DROPDOWN:
			{
				// Check the notify message origine (do nothing if it's not the search command button)
				if(lpnmTB->iItem != idCmdSearch) return FALSE;

				// Get the coordinates of the button
				RECT rc;
				SendMessage(lpnmTB->hdr.hwndFrom, TB_GETRECT, (WPARAM)lpnmTB->iItem, (LPARAM)&rc);

				// Convert to screen coordinates
				MapWindowPoints(lpnmTB->hdr.hwndFrom, HWND_DESKTOP, (LPPOINT)&rc, 2);

				// Create PopUp menu
				HMENU hMenu;
				hMenu = CreatePopupMenu();

				//
				char szDefaultName[80];
				long l = kPlugin.kFuncs->GetPreference(PREF_STRING, "kmeleon.general.searchEngineName", szDefaultName, (char*)"");

				//
				MENUITEMINFO mii = {0};
				char spref[100];
				char sname[80];

				// Fill the menu
				int i=0;
				for(; ; i++)
				{
					// Get search engine name
					sprintf(spref, "kmeleon.plugins.macros.search.engine%d.name", i);
					l = kPlugin.kFuncs->GetPreference(PREF_STRING, spref, sname, (char*)"");

					// Break the loop if name is null
					if( l==0 ) break;

					// Set item info
					mii.cbSize      = sizeof(MENUITEMINFO);
					mii.fMask       = MIIM_TYPE|MIIM_STATE|MIIM_ID;
					mii.fType       = MFT_STRING|MFT_RADIOCHECK;
					mii.fState      = !strcmp(sname,szDefaultName) ? MFS_DEFAULT|MFS_CHECKED : MFS_ENABLED;
					mii.dwTypeData  = (LPTSTR)sname;
					mii.cch         = strlen(sname);
					mii.hbmpChecked = NULL;
					mii.wID         = idCmdEngines;

					// Insert item into the menu
					InsertMenuItem(hMenu, i, true, &mii);
				}

				// Separator
				mii.cbSize = sizeof(MENUITEMINFO);
				mii.fMask  = MIIM_TYPE ;
				mii.fType  = MFT_SEPARATOR;

				InsertMenuItem(hMenu, i, true, &mii);

				// Item to configure the toolbar
				char s[8]={0};
				kPlugin.kFuncs->GetPreference(PREF_STRING, "general.useragent.locale", s, (char*)"");
				if(!strcmp(s,"fr-FR")) //french
				{
					strcpy(sname,"Proprits...");
				}
				else //default = english
				{
					strcpy(sname,"Settings...");
				}
				i++;
				mii.cbSize      = sizeof(MENUITEMINFO);
				mii.fMask       = MIIM_TYPE|MIIM_STATE|MIIM_ID;
				mii.fType       = MFT_STRING|MFT_RADIOCHECK;
				mii.fState      = !strcmp(sname,szDefaultName) ? MFS_DEFAULT|MFS_CHECKED : MFS_ENABLED;
				mii.dwTypeData  = (LPTSTR)sname;
				mii.cch         = strlen(sname);
				mii.hbmpChecked = NULL;
				mii.wID         = idCmdSettings;

				InsertMenuItem(hMenu, i, true, &mii);

				//Clear History
				strcpy(sname,"Clear History");

				i++;
				mii.cbSize      = sizeof(MENUITEMINFO);
				mii.fMask       = MIIM_TYPE|MIIM_STATE|MIIM_ID;
				mii.fType       = MFT_STRING|MFT_RADIOCHECK;
				mii.fState      = !strcmp(sname,szDefaultName) ? MFS_DEFAULT|MFS_CHECKED : MFS_ENABLED;
				mii.dwTypeData  = (LPTSTR)sname;
				mii.cch         = strlen(sname);
				mii.hbmpChecked = NULL;
				mii.wID         = idCmdClear;

				InsertMenuItem(hMenu, i, true, &mii);


				// Code taken from MSDN

				// Set up the popup menu.
				// Set rcExclude equal to the button rectangle so that if the toolbar
				// is too close to the bottom of the screen, the menu will appear above
				// the button rather than below it.
				TPMPARAMS tpm;
				tpm.cbSize    = sizeof(TPMPARAMS);
				tpm.rcExclude = rc;

				// Show the menu and wait for input.
				// If the user selects an item, its WM_COMMAND is sent.
				TrackPopupMenuEx(hMenu,
								TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_VERTICAL, // | TPM_RETURNCMD
								rc.left, rc.bottom, hWnd, &tpm);

				//
				DestroyMenu (hMenu);

				break;
			}
		}
	}


	//Select entries in response to changes(ComboBox)
	if(uMsg == WM_COMMAND & HIWORD(wParam) == CBN_SELENDOK)// & LOWORD(wParam) == IDCOMBO)
		{
		 	 char sTextA[1024];
			 PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWnd, _T("WNDCXT"));
			 COMBOBOXINFO info = { sizeof(COMBOBOXINFO) };
             GetComboBoxInfo(pcxt->hwndEdit, &info);
             HWND hWnd_cbx = info.hwndCombo;
             HWND hWnd_edit = info.hwndItem;
			 SendMessage(hWnd_cbx , CB_GETLBTEXT , ComboBox_GetCurSel(hWnd_cbx) , LPARAM(sTextA));  //Get current selected item
			 //MessageBox(hWnd_cbx , sTextA , NULL , 32);  //Test
			 SetWindowText(hWnd_edit , sTextA);
		     SendMessage(hWnd_edit , WM_CHAR, VK_RETURN, 0);
		}

	//Clear History
	if(uMsg == WM_COMMAND & LOWORD(wParam) == idCmdClear)
		{
			 PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWnd, _T("WNDCXT"));
			 COMBOBOXINFO info = { sizeof(COMBOBOXINFO) };
             GetComboBoxInfo(pcxt->hwndEdit, &info);
             HWND hWnd_cbx = info.hwndCombo;

		    int count = ComboBox_GetCount(hWnd_cbx);
			for(int i = 0; i < count; i ++)
			{
				 ComboBox_DeleteString(hWnd_cbx , 0);

			}

			//Delete sh.ini file
			DeleteFile((LPCTSTR)CurrentPath);
		}


	return CallWindowProc((WNDPROC)KMeleonWndProc, hWnd, uMsg, wParam, lParam);
}


/******************************************************************************
   EditProc

   Callback function for the EDIT control
 ******************************************************************************/
LRESULT CALLBACK EditProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static BOOL bFirst = FALSE; //for highlighting text

	switch(uMsg)
	{
		// Allow to treat RETURN keypress
		case WM_GETDLGCODE:
		{
			if ( lParam && ((LPMSG)lParam)->message == WM_KEYDOWN && ((LPMSG)lParam)->wParam == VK_RETURN )
				return DLGC_WANTALLKEYS;

			break;
		}

		// Check RETURN keypress
		case WM_CHAR:
		 {
			if (wParam == VK_RETURN)
			{
			   //Intercepting messages from other control (url bar)
			    if(!focus)
					{
					    SetWindowText(hWnd, "");
						SendMessage(hWnd , WM_KILLFOCUS , 0 ,0);
					    return 0;
					}

			   // Get search text
				char * sText;

				if (g_bUnicode)
				{
					WCHAR sTextW[1024];
					GetWindowTextW(hWnd , sTextW, sizeof(sTextW)/sizeof(WCHAR));

					sText = utf8_from_utf16(sTextW);

				}
				else
				{
					char sTextA[1024];
					GetWindowTextA(hWnd , sTextA, sizeof(sTextA));

					sText = utf8_from_ansi(sTextA);

				}

				// Perform search
				Search(sText);
                SetFocus(GetWindow(GetForegroundWindow() , GW_HWNDNEXT));

               //Add ComboBox Items
				char sItem[1024];
				GetWindowText(hWnd , sItem , sizeof(sItem));
				//To delete duplicate string
				//MessageBox(hWnd_edit , sItem , NULL , 32); //Test
				if(ComboBox_FindString(GetParent(hWnd) , 0 ,  LPCTSTR(sItem)) < 0 && Edit_GetTextLength(hWnd) > 0)
					{
				     SendMessage(GetParent(hWnd) , CB_INSERTSTRING , 0 , (LPARAM)sItem);
				     //Write ini file
				     int total = ComboBox_GetCount(GetParent(hWnd));
				     char szBuf[1024];
                     LPCTSTR total_t;
                     sprintf(szBuf , "%d" , total);
                     total_t = LPCTSTR(szBuf);
					 WritePrivateProfileString("Total" , "total" , total_t , (LPCTSTR)CurrentPath);  //Items total
					 WritePrivateProfileString("Item" , total_t , LPCTSTR(sItem) , (LPCTSTR)CurrentPath); //Add to ini file

				  }

				// Free memory
				free(sText);

				return 0;
			}

			break;
		}

		case WM_MOUSEWHEEL:
		{
				return 0;
				break;

		}

		// Editbox window gets the focus
		case WM_SETFOCUS:
		{

			focus = true;
		    HWND hWndActive = GetActiveWindow();
			// get context variables
			PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWndActive, _T("WNDCXT"));
			if(pcxt)
			{
				if(!pcxt->bText) //no text
				{
					pcxt->bText = TRUE;
					SetWindowText(hWnd, "");
				}

				bFirst = TRUE;
			}

			break;
		}

		// Left or right mouse button up --> hilight text
		case WM_LBUTTONUP:
		case WM_RBUTTONUP:
		{
			HWND hWndActive = GetActiveWindow();
			// get context variables
			PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWndActive, _T("WNDCXT"));
			if(pcxt)
			{
				// text is not empty
				if(pcxt->bText && bFirst)
				{
					// Get selection
					DWORD a, b;
					SendMessage(hWnd, EM_GETSEL, (WPARAM)&a, (LPARAM)&b);
					// If selection is null, we select all text
					if (a==b)
						SendMessage(hWnd, EM_SETSEL, (WPARAM)0, (LPARAM)-1); //does not work in WM_SETFOCUS

					bFirst = FALSE;
				}
			}

			break;
		}

		// Editbox window loses the focus
		case WM_KILLFOCUS:
		{
			 focus = false;
		    // Get search text
			CHAR szSearchText[1024];
			GetWindowText(hWnd, szSearchText, sizeof(szSearchText));

			// Empty search term
			if(!strlen(szSearchText))
			{
				HWND hWndActive = GetActiveWindow();
				// get context
				PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(hWndActive, _T("WNDCXT"));
				if(pcxt)
				{
					pcxt->bText = FALSE; //no text

					// do faded text
					char sValue[80];
					kPlugin.kFuncs->GetPreference(PREF_STRING, "kmeleon.general.searchEngineName", sValue, (char*)"");
					SetWindowText(pcxt->hwndEdit, sValue);
				}
			}

			break;
		}

		// paste & go ...
		case WM_PASTE:
		{
			BOOL bPasteAndGo = FALSE;
			kPlugin.kFuncs->GetPreference(PREF_BOOL, "kmeleon.plugins.searchbarext.pastego", &bPasteAndGo, &bPasteAndGo);
			if( bPasteAndGo )
			{
				HGLOBAL hglb;
				char *data;

				if (!OpenClipboard(hWnd))  break;

				hglb = IsWindowUnicode(hWnd) ? GetClipboardData(CF_UNICODETEXT) : GetClipboardData(CF_TEXT);
				if (hglb != NULL)
				{
					data = (char *)GlobalLock(hglb);

					if(IsWindowUnicode(hWnd))
						SetWindowTextW(hWnd, (WCHAR *)data);
					else
						SetWindowTextA(hWnd, (CHAR *)data);

					GlobalUnlock(hglb);
				}
				CloseClipboard();

				SendMessage(hWnd, WM_CHAR, VK_RETURN, 0);

				return TRUE;
			}
		}

	}

	return CallWindowProcUTF8((WNDPROC)g_wpEditProc, hWnd, uMsg, wParam, lParam);
}


/******************************************************************************
   ToolbarProc

   Callback function for the TOOLBAR control
 ******************************************************************************/
LRESULT CALLBACK ToolbarProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{

		//case WM_CHAR:
		//	MessageBox(hWnd , _T("dd") , NULL , 16);
		//	break;

		// Enabled-editbox childs are about to be drawn (only one here)
		case WM_CTLCOLOREDIT:
		{
			HDC  hdcEdit  = (HDC) wParam;
			HWND hwndEdit = (HWND) lParam;

			HWND hWndActive = GetActiveWindow();
			WNDCXT *pcxt = (WNDCXT *)GetProp(hWndActive, _T("WNDCXT"));
			if(pcxt)
			{

				if(hwndEdit == pcxt->hwndEdit)
				{
					SetTextColor( hdcEdit, (pcxt->bText)?GetSysColor(COLOR_WINDOWTEXT):RGB(128,128,128) );
					return (LRESULT)GetSysColorBrush(COLOR_WINDOW); //return bg color
				}
			}

			break;
		}


	}

	return CallWindowProc((WNDPROC)g_wpToolbarProc, hWnd, uMsg, wParam, lParam);
}


/******************************************************************************
   DialogProc
 ******************************************************************************/
BOOL APIENTRY DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_INITDIALOG:
		{
			// Get settings
			int iOpen=OPEN_IN_CUR_TAB;
			kPlugin.kFuncs->GetPreference(PREF_INT, "kmeleon.plugins.searchbarext.open", &iOpen, &iOpen);
			int iSize=200;
			kPlugin.kFuncs->GetPreference(PREF_INT, "kmeleon.plugins.searchbarext.size", &iSize, &iSize);
			BOOL bPasteAndGo = FALSE;
			kPlugin.kFuncs->GetPreference(PREF_BOOL, "kmeleon.plugins.searchbarext.pastego", &bPasteAndGo, &bPasteAndGo);

			//init.
			HWND htb = GetDlgItem(hDlg, ID_SIZE);
			SendMessage(htb, TBM_SETRANGE, TRUE, MAKELONG(50, 250));
			SendMessage(htb, TBM_SETPOS, TRUE, iSize);
			SetDlgItemInt(hDlg, ID_SIZETEXT, iSize, FALSE);

			//init. bPasteAndGo
			if(bPasteAndGo)
				CheckDlgButton(hDlg, ID_PASTEGO, BST_CHECKED);
			else
				CheckDlgButton(hDlg, ID_PASTEGO, BST_UNCHECKED);

			// Set dialogbox title --> version
			SetWindowText(hDlg, _T(PLUGIN_NAME " (" PLUGIN_VERSION ")"));

			// Manage locale --> default is english
			char s[8] = {0};
			kPlugin.kFuncs->GetPreference(PREF_STRING, "general.useragent.locale", s, (char*)"");
			if(!strcmp(s,"fr-FR")) //french
			{
				SetDlgItemText(hDlg, ID_OPEN, "Ouvrir le rsultat dans");
				SetDlgItemText(hDlg, ID_CUR_TAB, "l'onglet courant");
				SetDlgItemText(hDlg, ID_NEW_TAB, "un nouvel onglet");
				SetDlgItemText(hDlg, ID_NEW_TAB_BG, "en arrire plan");
				SetDlgItemText(hDlg, ID_NEW_WIN, "une nouvelle fentre");
				SetDlgItemText(hDlg, ID_NEW_WIN_BG, "en arrire plan");
				SetDlgItemText(hDlg, IDCANCEL, "Annuler");

				SetDlgItemText(hDlg, ID_RESIZE, "Taille de la zone de texte");

				SetDlgItemText(hDlg, ID_PASTEGO, "Validation auto. lors d'un copier/coller");
			}

			// Check setting "open in"
			switch(iOpen)
			{
				case OPEN_IN_CUR_TAB:
				default:
					CheckDlgButton(hDlg, ID_CUR_TAB, BST_CHECKED);
					break;
				case OPEN_IN_NEW_TAB:
					CheckDlgButton(hDlg, ID_NEW_TAB, BST_CHECKED);
					break;
				case OPEN_IN_NEW_TAB_BG:
					CheckDlgButton(hDlg, ID_NEW_TAB_BG, BST_CHECKED);
					break;
				case OPEN_IN_NEW_WIN:
					CheckDlgButton(hDlg, ID_NEW_WIN, BST_CHECKED);
					break;
				case OPEN_IN_NEW_WIN_BG:
					CheckDlgButton(hDlg, ID_NEW_WIN_BG, BST_CHECKED);
					break;
			}
			return TRUE;
		}

		case WM_COMMAND:
		{
			switch(wParam)
			{
				case IDOK:
				{
					int iOpen = OPEN_IN_CUR_TAB;
					if( IsDlgButtonChecked(hDlg, ID_NEW_TAB) )
						iOpen = OPEN_IN_NEW_TAB;
					else if( IsDlgButtonChecked(hDlg, ID_NEW_TAB_BG) )
						iOpen = OPEN_IN_NEW_TAB_BG;
					else if( IsDlgButtonChecked(hDlg, ID_NEW_WIN) )
						iOpen = OPEN_IN_NEW_WIN;
					else if( IsDlgButtonChecked(hDlg, ID_NEW_WIN_BG) )
						iOpen = OPEN_IN_NEW_WIN_BG;
					kPlugin.kFuncs->SetPreference(PREF_INT, "kmeleon.plugins.searchbarext.open", &iOpen, FALSE);

					BOOL bPasteAndGo = IsDlgButtonChecked(hDlg, ID_PASTEGO) ? TRUE: FALSE;
					kPlugin.kFuncs->SetPreference(PREF_BOOL, "kmeleon.plugins.searchbarext.pastego", &bPasteAndGo, FALSE);

					HWND htb = GetDlgItem(hDlg, ID_SIZE);
					int iSize = SendMessage(htb, TBM_GETPOS, 0, 0);
					kPlugin.kFuncs->SetPreference(PREF_INT, "kmeleon.plugins.searchbarext.size", &iSize, FALSE);

					RebuildBands(iSize);

					EndDialog(hDlg, 0);
					return TRUE;
				}
				case IDCANCEL:
				{
					EndDialog(hDlg, 0);
					return TRUE;
				}

					return TRUE;
			}
			break;
		}

		case WM_HSCROLL:
		{
			HWND htb = GetDlgItem(hDlg, ID_SIZE);
			SetDlgItemInt(hDlg, ID_SIZETEXT, SendMessage(htb, TBM_GETPOS, 0, 0), FALSE);
			return TRUE;
		}
	}

	return FALSE;
}


/******************************************************************************
   OpenURL
 ******************************************************************************/
void OpenURL(char *url, int open)
{
	if (kPlugin.kFuncs->GetKmeleonVersion() >= 0x01050000) //KM-1.5+
	{
		switch(open)
		{
			case OPEN_IN_NEW_TAB:
				kPlugin.kFuncs->NavigateTo(url, OPEN_NEWTAB, NULL);
				break;

			case OPEN_IN_NEW_TAB_BG:
				kPlugin.kFuncs->NavigateTo(url, OPEN_BACKGROUNDTAB, NULL);
				break;

			case OPEN_IN_NEW_WIN:
				kPlugin.kFuncs->NavigateTo(url, OPEN_NEW, NULL);
				break;

			case OPEN_IN_NEW_WIN_BG:
				kPlugin.kFuncs->NavigateTo(url, OPEN_BACKGROUND, NULL);
				break;

			case OPEN_IN_CUR_TAB:
			default:
				kPlugin.kFuncs->NavigateTo(url, OPEN_NORMAL, NULL);
				break;
		}
	}
	else
	{
		switch(open)
		{
			case OPEN_IN_NEW_TAB:
				kPlugin.kFuncs->SendMessage("layers", PLUGIN_NAME, "OpenURL", (LONG)url, (LONG)NULL);
				break;

			case OPEN_IN_NEW_TAB_BG:
				kPlugin.kFuncs->SendMessage("layers", PLUGIN_NAME, "OpenURLBg", (LONG)url, (LONG)NULL);
				break;

			case OPEN_IN_NEW_WIN:
				kPlugin.kFuncs->NavigateTo(url, OPEN_NEW, NULL);
				break;

			case OPEN_IN_NEW_WIN_BG:
				kPlugin.kFuncs->NavigateTo(url, OPEN_BACKGROUND, NULL);
				break;

			case OPEN_IN_CUR_TAB:
			default:
				kPlugin.kFuncs->NavigateTo(url, OPEN_NORMAL, NULL);
				break;
		}
	}
}

bool mustEscape(unsigned char c)
{
	static const char chars[]= " $&+,/:;=?@";
	if (c>127 || strchr(chars, c)) return true;
	return false;
}


unsigned urlencode(const char* str, char* dst)
{
	static const char hexChars[] = "0123456789ABCDEF";

	const char* p;

	if (!dst) {
		unsigned destLen = 0;
		for (p = str; *p; p++) {
			if ( !mustEscape( (unsigned char)(*p) ) || (*p)==' ' )
				destLen++;
			else
				destLen+=3;
		}
		return destLen+1;
	}

	unsigned di = 0;
	for (p = str; *p; p++) {
		register unsigned char c = (unsigned char)(*p);
		if ( !mustEscape(c) )
			dst[di++] = (*p);
		else if ((*p)==' ')
			dst[di++] = '+';
		else{
			dst[di++] = '%';
			dst[di++] = hexChars[c >> 4];
			dst[di++] = hexChars[c & 0x0f];
		}
	}
	dst[di] = 0;
	return di;
}




/******************************************************************************
   Search

   Perform a search.
 ******************************************************************************/
void Search(const char * sTerm)
{
	// Search string can be empty but not null
	if (!sTerm) return;

	// Get default search engine URL
	char sURL[1024] = {0};
	unsigned long l = kPlugin.kFuncs->GetPreference(PREF_STRING, "kmeleon.general.searchEngine", sURL, (char*)"");


	unsigned k = urlencode(sTerm, NULL);
	char* encodedTerm = (char*)malloc(k);
	urlencode(sTerm, encodedTerm);
	// Concatenate URL and search term
	strcat(sURL, encodedTerm);

	// Check setting
	int iOpen;
	kPlugin.kFuncs->GetPreference(PREF_INT, "kmeleon.plugins.searchbarext.open", &iOpen, &iOpen);

	// Open URL into a layer/window/tab
	OpenURL(sURL, iOpen);

	free(encodedTerm);
}


/******************************************************************************
   RefreshLayersEdit
 ******************************************************************************/
void RefreshLayersEdit(HWND hWnd, CHAR *sText)
{
	HWND tempHwnd = FindWindow(NULL, NULL);
	int i;
	DWORD procid;
	CHAR s[1024];

	// list all windows
	for(i=0; tempHwnd!=NULL; i++)
	{
		// do not care about current window
		if(tempHwnd != hWnd)
		{
			// check if window is K-M one
			PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(tempHwnd, _T("WNDCXT"));
			if(pcxt)
			{
				// Get search term
				GetWindowText(pcxt->hwndEdit, s, sizeof(s));

				// Search term is empty and (...)
				if( *s && !pcxt->bText )
					// Set editbox text to search engine name
					SetWindowText(pcxt->hwndEdit, sText);

			}
		}

		tempHwnd = GetWindow(tempHwnd, GW_HWNDNEXT);
	}

}


/******************************************************************************
   RebuildBands
 ******************************************************************************/
void RebuildBands(UINT uSize)
{
	HWND tempHwnd = FindWindow(NULL, NULL);
	int i;

	// list all windows
	for(i=0; tempHwnd!=NULL; i++)
	{
		// check if window is K-M one
		PTRWNDCXT pcxt = (PTRWNDCXT)GetProp(tempHwnd, _T("WNDCXT"));
		if(pcxt)
		{

			// Resize toolbar button (editbox)
			TBBUTTONINFO tbButtoninfo;
			ZeroMemory(&tbButtoninfo, sizeof(TBBUTTONINFO));
			tbButtoninfo.cbSize = sizeof(TBBUTTONINFO);
			tbButtoninfo.dwMask = TBIF_SIZE;
			tbButtoninfo.cx     = uSize;

			SendMessage(pcxt->hwndToolbar, TB_SETBUTTONINFO, idCmdEdit, (LPARAM)&tbButtoninfo);

			// Resize editbox
			int id = SendMessage(pcxt->hwndToolbar, TB_COMMANDTOINDEX, idCmdEdit, 0);

			RECT rc;
			SendMessage(pcxt->hwndToolbar, TB_GETITEMRECT, id, (LPARAM)&rc);

			MoveWindow(pcxt->hwndEdit, rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top + 200, TRUE);

			// Resize rebarband
			HWND hReBar = GetParent(pcxt->hwndToolbar);
			REBARBANDINFO rbbi;
			rbbi.cbSize = sizeof(REBARBANDINFO);
			rbbi.fMask  = RBBIM_CHILDSIZE | RBBIM_SIZE | RBBIM_IDEALSIZE;

			int iSize = 0;
			kPlugin.kFuncs->GetPreference(PREF_INT, "kmeleon.toolband.searchbarext.index", &iSize, &iSize);

			SendMessage(hReBar, RB_GETBANDINFO, (WPARAM) iSize, (LPARAM) &rbbi);

			rbbi.cxMinChild = rc.right-rc.left +40;
			rbbi.cxIdeal    = rbbi.cxMinChild;
			rbbi.cx         = rbbi.cxIdeal;

			SendMessage(hReBar, RB_SETBANDINFO, (WPARAM) iSize, (LPARAM) &rbbi);

			SendMessage(hReBar, RB_MINIMIZEBAND, (WPARAM) iSize, (LPARAM) 0);

		}

		tempHwnd = GetWindow(tempHwnd, GW_HWNDNEXT);
	}
}


/******************************************************************************
   ???
 ******************************************************************************/
extern "C" {
	KMELEON_PLUGIN kmeleonPlugin *GetKmeleonPlugin()
	{
		return &kPlugin;
	}
}






